package com.diagbot.facade;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.diagbot.client.UserServiceClient;
import com.diagbot.dto.LantoneProductDTO;
import com.diagbot.dto.ProductLineDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.UserOrgDTO;
import com.diagbot.entity.LantoneProduct;
import com.diagbot.entity.OpenedProducts;
import com.diagbot.entity.OpenedProductsIndex;
import com.diagbot.entity.wrapper.LantoneProductWrapper;
import com.diagbot.entity.wrapper.OpendProductWrapper;
import com.diagbot.enums.AccessTypeEnum;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.ProductAuditEnum;
import com.diagbot.enums.StatusEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.LantoneProductServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.AddProductsVO;
import com.diagbot.vo.LantoneProductSelectVO;
import com.diagbot.vo.OppendedProductVO;
import com.diagbot.vo.UpdateProductVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @Description:产品业务层
 * @author: wangyu
 * @time: 2018/9/19 15:52
 */
@Component
public class LantoneProductFacade extends LantoneProductServiceImpl {
    @Autowired
    private OpenedProductsFacade openedProductsFacade;
    @Autowired
    private UserServiceClient userServiceClient;
    @Autowired
    private OrderDetailsFacade orderDetailsFacade;

    /**
     * 产品线管理添加产品
     *
     * @param addProductsVO 需要提供的参数（产品名称，产品介绍，产品url，产品计费方式）
     * @return Boolean true
     */
    public Boolean addProducts(AddProductsVO addProductsVO) {
        LantoneProduct lantoneProduct = new LantoneProduct();
        BeanUtil.copyProperties(addProductsVO, lantoneProduct);
        lantoneProduct.setGmtCreate(DateUtil.now());
        lantoneProduct.setCreator(UserUtils.getCurrentPrincipleID());
        if (!save(lantoneProduct)) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR,
                    "产品添加失败");
        }
        return true;
    }


    /**
     * 朗通产品线管理修改产品
     *
     * @param updateProductVO 所需参数（产品名称，url，产品描述，接入模式，收费模式，是否试用）
     * @return Boolean true
     */
    public Boolean updateProduct(UpdateProductVO updateProductVO) {
        LantoneProduct lantoneProduct = this.getById(updateProductVO.getId());
        BeanUtil.copyProperties(updateProductVO, lantoneProduct);
        lantoneProduct.setModifier(UserUtils.getCurrentPrincipleID());
        lantoneProduct.setGmtModified(DateUtil.now());
        if (!updateById(lantoneProduct)) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR,
                    "产品修改失败");
        }
        return true;
    }

    /**
     * 删除产品业务逻辑
     *
     * @param updateProductVO 所需参数（产品id）
     * @return Boolean true
     */
    public Boolean deleteProduct(UpdateProductVO updateProductVO) {
        LantoneProduct lantoneProduct = getById(updateProductVO.getId());
        OpenedProducts openedProducts = new OpenedProducts();
        openedProducts.setProductId(updateProductVO.getId());
        openedProducts.setServiceStatus(StatusEnum.Enable.getKey());
        if (openedProductsFacade.getOpendById(openedProducts).size() != 0) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR,
                    "本产品还有用户使用，删除失败");
        }
        lantoneProduct.setModifier(UserUtils.getCurrentPrincipleID());
        lantoneProduct.setGmtModified(DateUtil.now());
        lantoneProduct.setIsDeleted(IsDeleteEnum.Y.getKey());
        if (!updateById(lantoneProduct)) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR,
                    "产品删除失败");
        }
        return true;
    }

    /**
     * 查询当前用户已开通产品
     *
     * @return 查询当前用户已开通产品
     */
    public List<LantoneProductWrapper> opendedProductByCurrentUser() {
        Long userId = Long.parseLong(UserUtils.getCurrentPrincipleID());
        List<LantoneProductWrapper> list = this.opendedProductByUserId(userId);
        List<Long> userIdList = new ArrayList<>();
        userIdList.add(userId);
        RespDTO<Map<Long, UserOrgDTO>> mapRespDTO = userServiceClient.getUserAndOrg(userIdList);
        if (mapRespDTO == null || !"0".equals(mapRespDTO.code)) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR,
                    "获取用户机构信息失败");
        }
        for (LantoneProductWrapper product : list) {
            product.setOrganizationName(mapRespDTO.data.get(userId).getOrgName());
            String[] accessType = product.getAccessType().split(",|，");
            String[] chargeType = product.getChargeType().split(",|，");
            String[] accessTypeArray = new String[accessType.length];
            String[] chargeTypeArray = new String[chargeType.length];

            for (int i = 0; i < accessType.length; i++) {
                accessTypeArray[i] = AccessTypeEnum.getName(Integer.parseInt(accessType[i]));
            }
            for (int i = 0; i < chargeType.length; i++) {
                chargeTypeArray[i] = AccessTypeEnum.getName(Integer.parseInt(chargeType[i]));
            }

            product.setAccessTypeArray(accessTypeArray);
            product.setChargeTypeArray(chargeTypeArray);

            String[] serviceIds = product.getServiceIds().split(",|，");
            Long[] services = new Long[serviceIds.length];
            for (int i = 0; i < serviceIds.length; i++) {
                services[i] = Long.parseLong(serviceIds[i]);
            }
            product.setServices(services);
        }
        return list;
    }

    /**
     * 根据用户id查询是否有开通产品
     *
     * @return 根据用户id查询是否有开通产品
     */
    public RespDTO<List<ProductLineDTO>> productLine() {
        List<ProductLineDTO> productLineDTOList = this.productLines();//所有产品
        Long userId = Long.parseLong(UserUtils.getCurrentPrincipleID());
        List<Long> opendId = openedProductsFacade.getOpendCountByUserId(userId);
        List<Long> orderDetailsId = orderDetailsFacade.getOrderDetailsByUser(userId);
        for (ProductLineDTO productLineDTO:productLineDTOList) {
            productLineDTO.setProductAudit(ProductAuditEnum.NotOpend.getKey());
            for (int i =0; i<opendId.size();i++){
                if(productLineDTO.getId()==opendId.get(i)){
                    productLineDTO.setProductAudit(ProductAuditEnum.Opended.getKey());
                }
            }
            for (int i =0; i<orderDetailsId.size();i++){
                if(productLineDTO.getId()==orderDetailsId.get(i)){
                    productLineDTO.setProductAudit(ProductAuditEnum.Audit.getKey());
                }
            }
        }
        return RespDTO.onSuc(productLineDTOList);
    }

    /**
     * 根据产品id查询当条产品线下所有订单信息
     *
     * @param oppendedProductVO 所需参数（产品id，页数，和每页条数，默认显示10条）
     * @return 当条产品线下所有订单信息
     */
    public IPage<OpendProductWrapper> opendedProduct(OppendedProductVO oppendedProductVO) {
        Page page =new Page();
        if(oppendedProductVO.getCurrent() !=0 && oppendedProductVO.getSize() != 0){
            page.setCurrent(oppendedProductVO.getCurrent());
            page.setSize(oppendedProductVO.getSize());
        }
        OpenedProductsIndex openedProductsIndex = new OpenedProductsIndex();
        BeanUtil.copyProperties(oppendedProductVO, openedProductsIndex);
        List<OpendProductWrapper> list = openedProductsFacade.getByProductId(page,openedProductsIndex).getRecords();
        List<Long> list1 = new ArrayList<>();
        for (OpendProductWrapper opendProducts : list) {
            Long longs = opendProducts.getUserId();
            list1.add(longs);
        }
        RespDTO<Map<Long, UserOrgDTO>> mapRespDTO = userServiceClient.getUserAndOrg(list1);
        if (mapRespDTO == null || !"0".equals(mapRespDTO.code)) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR,
                    "获取用户和机构信息失败");
        }
        Map<Long, UserOrgDTO> dataMap = mapRespDTO.data;
        for (OpendProductWrapper bean : list) {
            UserOrgDTO uo = dataMap.get(bean.getUserId());
            if (uo != null) {
                BeanUtil.copyProperties(uo,bean);
            }
        }
        return page;
    }

    /**
     * 用户-分页查询产品信息
     *
     * @param lantoneProductSelectVO 参数
     * @return 产品信息
     */
    public RespDTO<IPage<LantoneProductDTO>> index(LantoneProductSelectVO lantoneProductSelectVO) {
        Page page =new Page();
        if(lantoneProductSelectVO.getSize()==0){
            page.setSize(10l);
        }else {
            page.setSize(lantoneProductSelectVO.getSize());
        }
        if(lantoneProductSelectVO.getCurrent()==0){
            page.setCurrent(1l);
        }else {
            page.setCurrent(lantoneProductSelectVO.getCurrent());
        }
        IPage<LantoneProductDTO> res = this.selectProduct(page, lantoneProductSelectVO.getName());
        return RespDTO.onSuc(res);
    }
}
